@@ -1,15 +1,22 @@
 # -*- cperl -*-
-# Copyright (c) 2004, 2018, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2004, 2021, Oracle and/or its affiliates.
 # 
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; version 2 of the License.
-# 
+# it under the terms of the GNU General Public License, version 2.0,
+# as published by the Free Software Foundation.
+#
+# This program is also distributed with certain software (including
+# but not limited to OpenSSL) that is licensed under separate terms,
+# as designated in a particular file or component or in included license
+# documentation.  The authors of MySQL hereby grant you an additional
+# permission to link the program and your derivative works with the
+# separately licensed software that they have included with MySQL.
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-# 
+# GNU General Public License, version 2.0, for more details.
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
@@ -31,6 +38,7 @@ our @EXPORT= qw(report_option mtr_print_line mtr_print_thick_line
 
 use mtr_match;
 use File::Spec;
+use My::Constants;
 use My::Platform;
 use POSIX qw(_exit floor);
 use IO::Handle qw[ flush ];
@@ -544,7 +552,9 @@ sub mtr_report_stats ($$;$) {
     {
       # Test was skipped (disabled not counted)
       $tot_skipped++ unless $tinfo->{'disable'};
-      $tot_skipdetect++ if $tinfo->{'skip_detected_by_test'};
+      $tot_skipdetect++
+        if (defined $tinfo->{'skip_reason'}
+            and $tinfo->{skip_reason} eq MTR_SKIP_BY_TEST);
     }
     elsif ( $tinfo->{'result'} eq 'MTR_RES_PASSED' )
     {
