@@ -1,16 +1,23 @@
 #ifndef ITEM_FUNC_INCLUDED
 #define ITEM_FUNC_INCLUDED
 
-/* Copyright (c) 2000, 2017, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -217,7 +224,7 @@ public:
   virtual Item *key_item() const { return args[0]; }
   virtual bool const_item() const { return const_item_cache; }
   inline Item **arguments() const
-  { DBUG_ASSERT(argument_count() > 0); return args; }
+  { assert(argument_count() > 0); return args; }
   /**
     Copy arguments from list to args array
 
@@ -391,7 +398,7 @@ public:
 
   bool has_timestamp_args()
   {
-    DBUG_ASSERT(fixed == TRUE);
+    assert(fixed == TRUE);
     for (uint i= 0; i < arg_count; i++)
     {
       if (args[i]->type() == Item::FIELD_ITEM &&
@@ -403,7 +410,7 @@ public:
 
   bool has_date_args()
   {
-    DBUG_ASSERT(fixed == TRUE);
+    assert(fixed == TRUE);
     for (uint i= 0; i < arg_count; i++)
     {
       if (args[i]->type() == Item::FIELD_ITEM &&
@@ -416,7 +423,7 @@ public:
 
   bool has_time_args()
   {
-    DBUG_ASSERT(fixed == TRUE);
+    assert(fixed == TRUE);
     for (uint i= 0; i < arg_count; i++)
     {
       if (args[i]->type() == Item::FIELD_ITEM &&
@@ -429,7 +436,7 @@ public:
 
   bool has_datetime_args()
   {
-    DBUG_ASSERT(fixed == TRUE);
+    assert(fixed == TRUE);
     for (uint i= 0; i < arg_count; i++)
     {
       if (args[i]->type() == Item::FIELD_ITEM &&
@@ -555,7 +562,7 @@ public:
   String *val_str(String*str);
   my_decimal *val_decimal(my_decimal *decimal_value);
   longlong val_int()
-    { DBUG_ASSERT(fixed == 1); return (longlong) rint(val_real()); }
+  { assert(fixed == 1); return (longlong) rint(val_real()); }
   bool get_date(MYSQL_TIME *ltime, my_time_flags_t fuzzydate)
   {
     return get_date_from_real(ltime, fuzzydate);
@@ -666,11 +673,11 @@ public:
 
   void fix_num_length_and_dec();
   void find_num_type();
-  String *str_op(String *str) { DBUG_ASSERT(0); return 0; }
+  String *str_op(String *str) { assert(0); return 0; }
   bool date_op(MYSQL_TIME *ltime, my_time_flags_t fuzzydate)
-  { DBUG_ASSERT(0); return 0; }
+  { assert(0); return 0; }
   bool time_op(MYSQL_TIME *ltime)
-  { DBUG_ASSERT(0); return 0; }
+  { assert(0); return 0; }
 };
 
 
@@ -690,11 +697,11 @@ class Item_num_op :public Item_func_numhybrid
   }
 
   void find_num_type();
-  String *str_op(String *str) { DBUG_ASSERT(0); return 0; }
+  String *str_op(String *str) { assert(0); return 0; }
   bool date_op(MYSQL_TIME *ltime, my_time_flags_t fuzzydate)
-  { DBUG_ASSERT(0); return 0; }
+  { assert(0); return 0; }
   bool time_op(MYSQL_TIME *ltime)
-  { DBUG_ASSERT(0); return 0; }
+  { assert(0); return 0; }
 };
 
 #endif /* MYSQL_SERVER */
@@ -765,7 +772,7 @@ public:
   const char *func_name() const { return "connection_id"; }
   void fix_length_and_dec();
   bool fix_fields(THD *thd, Item **ref);
-  longlong val_int() { DBUG_ASSERT(fixed == 1); return value; }
+  longlong val_int() { assert(fixed == 1); return value; }
   bool check_gcol_func_processor(uchar *int_arg) { return true;}
 };
 
@@ -898,7 +905,7 @@ class Item_func_div :public Item_num_op
 public:
   uint prec_increment;
   Item_func_div(const POS &pos, Item *a,Item *b) :Item_num_op(pos, a,b) {}
-  longlong int_op() { DBUG_ASSERT(0); return 0; }
+  longlong int_op() { assert(0); return 0; }
   double real_op();
   my_decimal *decimal_op(my_decimal *);
   const char *func_name() const { return "/"; }
@@ -1448,7 +1455,7 @@ class Item_func_bit_length :public Item_func_length
 public:
   Item_func_bit_length(const POS &pos, Item *a) :Item_func_length(pos, a) {}
   longlong val_int()
-    { DBUG_ASSERT(fixed == 1); return Item_func_length::val_int()*8; }
+  { assert(fixed == 1); return Item_func_length::val_int()*8; }
   const char *func_name() const { return "bit_length"; }
 };
 
@@ -1742,7 +1749,7 @@ public:
   enum Functype functype() const   { return UDF_FUNC; }
   bool fix_fields(THD *thd, Item **ref)
   {
-    DBUG_ASSERT(fixed == 0);
+    assert(fixed == 0);
     bool res= udf.fix_fields(thd, this, arg_count, args);
     used_tables_cache= udf.used_tables_cache;
     const_item_cache= udf.const_item_cache;
@@ -1819,7 +1826,7 @@ class Item_func_udf_float :public Item_udf_func
   {}
   longlong val_int()
   {
-    DBUG_ASSERT(fixed == 1);
+    assert(fixed == 1);
     return (longlong) rint(Item_func_udf_float::val_real());
   }
   my_decimal *val_decimal(my_decimal *dec_buf)
@@ -2268,12 +2275,12 @@ public:
   my_decimal *val_decimal(my_decimal *decimal_buffer);
   bool get_date(MYSQL_TIME *ltime, my_time_flags_t fuzzydate)
   {
-    DBUG_ASSERT(0);
+    assert(0);
     return true;
   }
   bool get_time(MYSQL_TIME *ltime)
   {
-    DBUG_ASSERT(0);
+    assert(0);
     return true;
   }
 
@@ -2302,6 +2309,7 @@ class Item_func_get_system_var :public Item_var_func
   my_bool cached_null_value;
   query_id_t used_query_id;
   uchar cache_present;
+  Sys_var_tracker var_tracker;
 
 public:
   Item_func_get_system_var(sys_var *var_arg, enum_var_type var_type_arg,
@@ -2322,15 +2330,6 @@ public:
   { return val_decimal_from_real(dec_buf); }
   /* TODO: fix to support views */
   const char *func_name() const { return "get_system_var"; }
-  /**
-    Indicates whether this system variable is written to the binlog or not.
-
-    Variables are written to the binlog as part of "status_vars" in
-    Query_log_event, as an Intvar_log_event, or a Rand_log_event.
-
-    @return true if the variable is written to the binlog, false otherwise.
-  */
-  bool is_written_to_binlog();
   bool eq(const Item *item, bool binary_cmp) const;
 
   void cleanup();
@@ -2400,7 +2399,7 @@ public:
   bool fix_fields(THD *thd, Item **ref);
   bool eq(const Item *, bool binary_cmp) const;
   /* The following should be safe, even if we compare doubles */
-  longlong val_int() { DBUG_ASSERT(fixed == 1); return val_real() != 0.0; }
+  longlong val_int() { assert(fixed == 1); return val_real() != 0.0; }
   double val_real();
   virtual void print(String *str, enum_query_type query_type);
 
@@ -2419,8 +2418,8 @@ public:
    */
   ulonglong get_count()
   {
-    DBUG_ASSERT(ft_handler);
-    DBUG_ASSERT(table_ref->table->file->ha_table_flags() & HA_CAN_FULLTEXT_EXT);
+    assert(ft_handler);
+    assert(table_ref->table->file->ha_table_flags() & HA_CAN_FULLTEXT_EXT);
 
     return ((FT_INFO_EXT *)ft_handler)->could_you->
       count_matches((FT_INFO_EXT *)ft_handler);
@@ -2434,14 +2433,14 @@ public:
    */
   bool ordered_result()
   {
-    DBUG_ASSERT(!master);
+    assert(!master);
     if (hints->get_flags() & FT_SORTED)
       return true;
 
     if ((table_ref->table->file->ha_table_flags() & HA_CAN_FULLTEXT_EXT) == 0)
       return false;
 
-    DBUG_ASSERT(ft_handler);
+    assert(ft_handler);
     return ((FT_INFO_EXT *)ft_handler)->could_you->get_flags() & 
       FTS_ORDERED_RESULT;
   }
@@ -2454,7 +2453,7 @@ public:
    */
   bool docid_in_result()
   {
-    DBUG_ASSERT(ft_handler);
+    assert(ft_handler);
 
     if ((table_ref->table->file->ha_table_flags() & HA_CAN_FULLTEXT_EXT) == 0)
       return false;
@@ -2498,7 +2497,7 @@ public:
   */
   Ft_hints *get_hints()
   {
-    DBUG_ASSERT(!master);
+    assert(!master);
     return hints;
   }
 
@@ -2510,7 +2509,7 @@ public:
   */
   void set_hints_op(enum ft_operation type, double value_arg)
   {
-    DBUG_ASSERT(!master);
+    assert(!master);
     hints->set_hint_op(type, value_arg);
   }
   
@@ -2527,7 +2526,7 @@ public:
   */
   bool can_skip_ranking()
   {
-    DBUG_ASSERT(!master);
+    assert(!master);
     return (!(hints->get_flags() & FT_SORTED) && // FT_SORTED is no set
             used_in_where_only &&                // MATCH result is not used
                                                  // in expression
@@ -2541,7 +2540,7 @@ public:
   */
   void set_simple_expression(bool val)
   {
-    DBUG_ASSERT(!master);
+    assert(!master);
     simple_expression= val;
   }
 
@@ -2553,7 +2552,7 @@ public:
   */
   bool is_simple_expression()
   {
-    DBUG_ASSERT(!master);
+    assert(!master);
     return simple_expression;
   }
 
@@ -2599,7 +2598,7 @@ private:
     if (!(flags & FT_BOOL))
       return false;
 
-    DBUG_ASSERT(tr && tr->file);
+    assert(tr && tr->file);
 
     // Assume that if extended fulltext API is not supported,
     // non-indexed columns are allowed.  This will be true for MyISAM.
@@ -2816,6 +2815,22 @@ public:
   }
 
   virtual void update_null_value();
+
+  /**
+    Ensure that deterministic functions are not evaluated in preparation phase
+    by returning false before tables are locked and true after they are locked.
+    (can_be_evaluated_now() handles this because a function has the
+    has_subquery() property).
+
+     @retval true if tables are locked for deterministic functions
+     @retval false Otherwise
+  */
+  bool const_item() const
+  {
+    if (used_tables() == 0)
+      return can_be_evaluated_now();
+    return false;
+  }
 };
 
 
@@ -2870,7 +2885,7 @@ public:
 
 
 Item *get_system_var(Parse_context *pc, enum_var_type var_type, LEX_STRING name,
-                     LEX_STRING component);
+                     LEX_STRING component, bool unsafe);
 extern bool check_reserved_words(LEX_STRING *name);
 extern enum_field_types agg_field_type(Item **items, uint nitems);
 double my_double_round(double value, longlong dec, bool dec_unsigned,
