@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2016, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -39,9 +39,8 @@
   flush_io_cache().  
 */
 
-#include "sql_priv.h"
-#include "sql_class.h"                          // THD
 #ifdef HAVE_REPLICATION
+#include "sql_class.h"                          // THD
 
 extern "C" {
 
@@ -59,7 +58,7 @@ int _my_b_net_read(IO_CACHE *info, uchar *Buffer,
 		   size_t Count MY_ATTRIBUTE((unused)))
 {
   ulong read_length;
-  NET *net= &(current_thd)->net;
+  NET *net= current_thd->get_protocol_classic()->get_net();
   DBUG_ENTER("_my_b_net_read");
 
   if (!info->end_of_file)
@@ -76,7 +75,7 @@ int _my_b_net_read(IO_CACHE *info, uchar *Buffer,
     DBUG_RETURN(1);
   }
   /* to set up stuff for my_b_get (no _) */
-  info->read_end = (info->read_pos = (uchar*) net->read_pos) + read_length;
+  info->read_end = (info->read_pos = net->read_pos) + read_length;
   Buffer[0] = info->read_pos[0];		/* length is always 1 */
 
   /*
