@@ -1,4 +1,4 @@
-/* Copyright (c) 2007, 2015, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2007, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -64,7 +64,7 @@ int fill_query_profile_statistics_info(THD *thd, TABLE_LIST *tables,
   const char *old= thd->lex->sql_command == SQLCOM_SHOW_PROFILE ?
                      "SHOW PROFILE" : "INFORMATION_SCHEMA.PROFILING";
 
-  DBUG_ASSERT(thd->lex->sql_command != SQLCOM_SHOW_PROFILES);
+  assert(thd->lex->sql_command != SQLCOM_SHOW_PROFILES);
 
   push_deprecated_warn(thd, old, "Performance Schema");
   return(thd->profiling.fill_statistics_info(thd, tables, cond));
@@ -214,7 +214,7 @@ void PROF_MEASUREMENT::set_label(const char *status_arg,
 
   allocated_status_memory= (char *) my_malloc(key_memory_PROFILE,
                                               sizes[0] + sizes[1] + sizes[2], MYF(0));
-  DBUG_ASSERT(allocated_status_memory != NULL);
+  assert(allocated_status_memory != NULL);
 
   cursor= allocated_status_memory;
 
@@ -319,7 +319,7 @@ void QUERY_PROFILE::set_query_source(const char *query_source_arg,
   /* Truncate to avoid DoS attacks. */
   size_t length= min(MAX_QUERY_LENGTH, query_length_arg);
 
-  DBUG_ASSERT(m_query_source.str == NULL); /* we don't leak memory */
+  assert(m_query_source.str == NULL); /* we don't leak memory */
   if (query_source_arg != NULL)
   {
     m_query_source.str= my_strndup(key_memory_PROFILE,
@@ -335,7 +335,7 @@ void QUERY_PROFILE::new_status(const char *status_arg,
   PROF_MEASUREMENT *prof;
   DBUG_ENTER("QUERY_PROFILE::status");
 
-  DBUG_ASSERT(status_arg != NULL);
+  assert(status_arg != NULL);
 
   if ((function_arg != NULL) && (file_arg != NULL))
     prof= new PROF_MEASUREMENT(this, status_arg, function_arg, base_name(file_arg), line_arg);
@@ -421,7 +421,7 @@ void PROFILING::start_new_query(const char *initial_state)
 
   if (! enabled) DBUG_VOID_RETURN;
 
-  DBUG_ASSERT(current == NULL);
+  assert(current == NULL);
   current= new QUERY_PROFILE(this, initial_state);
 
   DBUG_VOID_RETURN;
@@ -573,8 +573,8 @@ static void my_b_print_status(IO_CACHE *log_file, const char *status,
                               PROF_MEASUREMENT *start, PROF_MEASUREMENT *stop)
 {
   DBUG_ENTER("my_b_print_status");
-  DBUG_ASSERT(log_file != NULL);
-  DBUG_ASSERT(status != NULL);
+  assert(log_file != NULL);
+  assert(status != NULL);
   char query_time_buff[22+7];
   const char *tmp;
 
