@@ -1,4 +1,4 @@
-/* Copyright (c) 2005, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2005, 2021, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -224,7 +224,7 @@ int ha_blackhole::rnd_pos(uchar * buf, uchar *pos)
   DBUG_ENTER("ha_blackhole::rnd_pos");
   MYSQL_READ_ROW_START(table_share->db.str, table_share->table_name.str,
                        FALSE);
-  DBUG_ASSERT(0);
+  assert(0);
   MYSQL_READ_ROW_DONE(0);
   DBUG_RETURN(0);
 }
@@ -233,7 +233,7 @@ int ha_blackhole::rnd_pos(uchar * buf, uchar *pos)
 void ha_blackhole::position(const uchar *record)
 {
   DBUG_ENTER("ha_blackhole::position");
-  DBUG_ASSERT(0);
+  assert(0);
   DBUG_VOID_RETURN;
 }
 
